static unsigned int size_x = 80;
static unsigned int size_y = 50;
static volatile unsigned short *vram_addr = (unsigned short *) 0xb00a0000;
static unsigned short cursor_x = 0;
static unsigned short cursor_y = 0;
static int console_needs_init = 1;

void
init_console(void)
{
    size_x = 80;
    size_y = 50;
    vram_addr = (unsigned short *) 0xb00a0000;
    cursor_x = 0;
    cursor_y = 0;
}

void
set_size_x(unsigned int x)
{
	size_x = x;
}

void
set_size_y(unsigned int y)
{
	size_y = y;
}

void
set_vram(unsigned short *vram)
{
	vram_addr = vram;
}

void
set_cursor(unsigned int x, unsigned int y)
{
	cursor_x = x;
	cursor_y = y;
}

void
print_char(unsigned int x, unsigned int y, unsigned char c)
{
	volatile unsigned short *caddr;

	caddr = vram_addr + (cursor_y * size_x) + cursor_x;
	*caddr = 0x0f00 | c;
}

static void
scroll()
{
	volatile unsigned short *caddr;
	register int i;

	caddr = vram_addr;
	for(i=0;i<size_x * (size_y-1);i++) {
		*caddr = *(caddr + size_x);
		caddr++;
		}

	caddr = vram_addr + (size_x * (size_y-1));
	for(i=0;i<size_x;i++)
		*caddr++ = 0x0f00 | (unsigned short) ' ';
}

void
print_string(const unsigned char *str)
{
	unsigned char c;

    if(console_needs_init == 1) {
        init_console();
        console_needs_init = 0;
    }
	while(c = *str++)
		switch(c) {
			case '\n':
				cursor_x = 0;
				cursor_y += 1;
				if(cursor_y >= size_y)
					scroll();
				break;
			default:
				print_char(cursor_x, cursor_y, c);
				cursor_x += 1;
				if(cursor_x >= size_x) {
					cursor_x = 0;
					cursor_y += 1;
					if(cursor_y >= size_y)
						scroll();
					}
				break;
			}
}
