/*
 * get_nlist.c -- This file contains the routines for getting a symbol
 *		  table entry when booting the Amiga Linux kernel.
 *
 * Copyright 1993 by Hamish Macdonald
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file README.legal in the main directory of this archive
 * for more details.
 *
 */


#include <arc/stdlib.h>
#include <sys/fcntl.h>
#include <sys/file.h>
#include <linux/a.out.h>
#include <linux/string.h>

extern long memptr;
extern struct exec kexec;

struct nlist *get_nlist (const char *fname, const char *symname)
{
    int fd;
    struct exec ex;
    struct nlist *nl, *p = NULL, *syms;
    char *strs;
    off_t filesize;
    long strsize, numsyms;

    fd = open (fname, O_RDONLY);
    if (fd == -1)
	return NULL;

#ifdef DEBUG
    printf ("fd is %d\r\n", fd);
#endif

    read (fd, &ex, sizeof(ex));
    if (!ex.a_syms) {
	close (fd);
	return NULL;
    }

#ifdef DEBUG
    printf ("%d bytes of symbol data\r\n", ex.a_syms);
#endif

#if 0
printf("%s %u\r\n", __FILE__, __LINE__); dumpmalloc();
printf("%x\r\n", memptr);
printf("%x %x\r\n", memptr + kexec.a_text, kexec.a_data);
#endif
    syms = (struct nlist *)malloc (ex.a_syms);
    if (!syms) {
	close (fd);
	return NULL;
    }
#if 0
printf("%s %u\r\n", __FILE__, __LINE__); dumpmalloc();
printf("%x\r\n", memptr);
printf("%x %x\r\n", memptr + kexec.a_text, kexec.a_data);
#endif
    lseek (fd, N_SYMOFF(ex), L_SET);
#if 0
printf("%s %u\r\n", __FILE__, __LINE__);
printf("%x\r\n", memptr);
printf("syms: 0x%x 0x%x\r\n", syms, ex.a_syms);
#endif
    read (fd, syms, ex.a_syms);
#if 0
printf("%s %u\r\n", __FILE__, __LINE__);
printf("%x\r\n", memptr);
printf("%x %x\r\n", memptr + kexec.a_text, kexec.a_data);
#endif
    numsyms = ex.a_syms / sizeof (struct nlist);
#ifdef DEBUG
    printf ("there are %d symbols\r\n", numsyms);
#endif
    filesize = lseek (fd, 0L, L_XTND);
#ifdef DEBUG
    printf ("there are %d characters in the file\r\n", filesize);
#endif
    strsize = filesize - N_STROFF(ex);
#ifdef DEBUG
    printf ("%d characters in the string table\r\n", strsize);
#endif

#if 0
printf("%s %u\r\n", __FILE__, __LINE__);
printf("%x\r\n", memptr);
printf("%x %x\r\n", memptr + kexec.a_text, kexec.a_data);
#endif
    strs = (char *)malloc (strsize);
#if 0
printf("%s %u\r\n", __FILE__, __LINE__);
printf("%x\r\n", memptr);
printf("%x %x\r\n", memptr + kexec.a_text, kexec.a_data);
#endif
    if (!strs) {
	free (syms);
	close (fd);
	return NULL;
    }
#if 0
/* Weiter vorne */
printf("%s %u\r\n", __FILE__, __LINE__);
printf("%x\r\n", memptr);
printf("%x %x\r\n", memptr + kexec.a_text, kexec.a_data);
#endif
    lseek (fd, N_STROFF(ex), L_SET);
#if 0
/* Weiter vorne */
printf("%s %u\r\n", __FILE__, __LINE__);
printf("%x\r\n", memptr);
printf("%x %x\r\n", memptr + kexec.a_text, kexec.a_data);
#endif
    read (fd, strs, strsize);

#if 0
/* Weiter vorne */
printf("%s %u\r\n", __FILE__, __LINE__);
printf("%x\r\n", memptr);
printf("%x %x\r\n", memptr + kexec.a_text, kexec.a_data);
#endif

    for (nl = syms; nl < syms + numsyms; nl++) {
#ifdef DEBUG
	printf ("checking symbol number %d, name %s\r\n",
		nl - syms, strs + nl->n_un.n_strx);
#endif
	if (strcmp (symname, strs + nl->n_un.n_strx) == 0
	    && nl->n_type == N_BSS | N_EXT) {
	    p = (struct nlist *)malloc (sizeof (struct nlist)
					+ strlen(strs + nl->n_un.n_strx) + 1);
	    if (!p)
		break;
	    *p = *nl;
	    p->n_un.n_name = (char *)(p+1);
	    strcpy (p->n_un.n_name, strs + nl->n_un.n_strx);
	}
    }

    free (strs);
    free (syms);
    close (fd);
    return p;
}
