/*
 * include/asm-mips/mipsregs.h
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 1994 by Ralf Baechle
 */

#ifndef _ASM_MIPS_MIPSREGS_H_
#define _ASM_MIPS_MIPSREGS_H_

/*
 * The following macros are especially useful for __asm__
 * inline assembler.
 */

#ifndef __STR
#define __STR(x) #x
#endif
#ifndef STR
#define STR(x) __STR(x)
#endif

/*
 * On the R2000/3000 load instructions are not interlocked -
 * we therefore sometimes need to fill load delay slots with nop
 * which are not needed for >=R4000.
 *
 * FIXME: Don't know about R6000
 */
#if !defined (__R4000__)
#define FILL_LDS nop
#else
#define FILL_LDS
#endif

/*
 * Coprocessor 0 register names
 */
#define CP0_INDEX $0
#define CP0_RANDOM $1
#define CP0_ENTRYLO0 $2
#define CP0_ENTRYLO1 $3
#define CP0_CONTEXT $4
#define CP0_PAGEMASK $5
#define CP0_WIRED $6
#define CP0_BADVADDR $8
#define CP0_COUNT $9
#define CP0_ENTRYHI $10
#define CP0_COMPARE $11
#define CP0_STATUS $12
#define CP0_CAUSE $13
#define CP0_EPC $14
#define CP0_PRID $15
#define CP0_CONFIG $16
#define CP0_LLADDR $17
#define CP0_WATCHLO $18
#define CP0_WATCHHI $19
#define CP0_XCONTEXT $20
#define CP0_ECC $26
#define CP0_CACHEERR $27
#define CP0_TAGLO $28
#define CP0_TAGHI $29
#define CP0_ERROREPC $30

/*
 * Values for pagemask register
 */
#define PM_4K   0x00000000
#define PM_16K  0x00006000
#define PM_64K  0x0001e000
#define PM_256K 0x0007e000
#define PM_1M   0x001fe000
#define PM_4M   0x007fe000
#define PM_16M  0x01ffe000

/*
 * Values used for computation of new tlb entries
 */
#define PL_4K   12
#define PL_16K  14
#define PL_64K  16
#define PL_256K 18
#define PL_1M   20
#define PL_4M   22
#define PL_16M  24

/*
 * Compute a vpn/pfn entry for EntryHi register
 */
#define VPN(addr,pagesizeshift) ((addr) & ~((1 << (pagesizeshift))-1))
#define PFN(addr,pagesizeshift) (((addr) & ((1 << (pagesizeshift))-1)) << 6)

/*
 * Macros to access the system control copprocessor
 */
#define read_32bit_cp0_register(source)                                        \
({ int __res;                                                                  \
        __asm__ __volatile__(                                                  \
        "mfc0\t%0,"STR(source)                                                 \
        : "=d" (__res));                                                       \
        __res;})

#define read_64bit_cp0_register(target,source)                                 \
        __asm__ __volatile__(                                                  \
        ".set\tnoat\n\t"                                                       \
        "dmfc0\t$1,"STR(source)"\n\t"                                          \
        "sd\t$1,(%0)\n\t"                                                      \
        ".set\tat"                                                             \
        :                                                                      \
        : "d" (target)                                                         \
        : "$1","memory");


#define write_32bit_cp0_register(register,value)                               \
        __asm__ __volatile__(                                                  \
        "mtc0\t%0,"STR(register)                                               \
        : : "d" (value));

/*
 * Inline code for use of the ll and sc instructions
 *
 * FIXME: This code code will break on R[23]00 CPUs
 */

#define load_linked(addr)                                                      \
({                                                                             \
	unsigned int __res;                                                    \
                                                                               \
	__asm__ __volatile__(                                                  \
	"ll\t%0,(%1)"                                                          \
	: "=d" (__res)                                                         \
	: "d" ((unsigned int) (addr)));                                        \
                                                                               \
	__res;                                                                 \
})

#define store_conditional(value,addr)                                          \
({                                                                             \
	int	__res;                                                         \
                                                                               \
	__asm__ __volatile__(                                                  \
	"sc\t%0,(%2)"                                                          \
	: "=d" (__res)                                                         \
	: "0" (value), "d" (addr));                                            \
                                                                               \
	__res;                                                                 \
})

#endif /* _ASM_MIPS_MIPSREGS_H_ */
